<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\FPA\ObrigacaoLegal\CNAB\PercorreCNAB;
use HCM\FPA\ObrigacaoLegal\CNAB\Banestes\Cnab021;
use HCM\FPA\ObrigacaoLegal\CNAB\Itau\Cnab341;
use HCM\FPA\ObrigacaoLegal\CNAB\Caixa\Cnab104;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab240;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab200;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab240237;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab200237;

class CNABLeituraController extends Crud {

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, Connection $connection, \HCM\FPA\Services\CNABLeituraService $cnabLeituraService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->connection = $connection;        
        $this->cnabLeituraService = $cnabLeituraService;
        parent::__construct($dataSourceManager);
    }

    public function leituraCNAB (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $data = array();
            $message = array();
            foreach ($row['DSPATH'] as $arquivo) {
                $row['DSPATH'] = [$arquivo];

                switch($row['BANCO']) {
                    case 'BRADESCO':
                        $data = $this->cnabLeituraService->leituraArquivoBradesco($row);
                        break;
                    default:
                        $message = new \Zeedhi\Framework\DTO\Response\Message('Banco inválido', 'E');
                        $response->addMessage($message);
                        break;
                }
            }
            if(array_key_exists('erro', $message)) {
                $message = new \Zeedhi\Framework\DTO\Response\Message($message['erro'], 'E');
                $response->addMessage($message);
            }
            if($data['sucesso']) {
                $message = new \Zeedhi\Framework\DTO\Response\Message($data['sucesso'], 'S');
                $response->addMessage($message);
                unset($data['sucesso']);

                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('data', $data));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}