<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\FPA\Calculos\Adiantamento\PercorreAdiantamento;
use HCM\FPA\Calculos\Folha\PercorreFolha;
use HCM\FPA\Calculos\FolhaComplementar\PercorreFolhaComplementar;
use HCM\FPA\Calculos\PLR\PercorrePLR;
use HCM\FPA\Calculos\DecimoTerceiro\PercorreDecimoTerceiro;
use HCM\FPA\Calculos\Ferias\PercorreFerias;
use HCM\FPA\Calculos\Fechamento\PercorreFechamento;
use HCM\FPA\Calculos\Rescisao\PercorreRescisao;
use HCM\FPA\Calculos\Provisao\PercorreProvisao;
use HCM\FPA\Calculos\Provisao\PercorreProvisaoOtimizada;
use HCM\FPA\Calculos\Apolo\PreparaFilaApolo;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Controllers\LogRetrocessoCalculoController;
use HCM\Geral\Services\ESocialService;

class CalculoController extends Crud {
    private static $NM_PROCESSAMENTO = array('CADGOZOFERIAS' => 'Cadastro de Gozo de Férias');
    private $logRetrocessoCalculoController;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, LogRetrocessoCalculoController $logRetrocessoCalculoController, ESocialService $eSocialService){
        parent::__construct($dataSourceManager);
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->logRetrocessoCalculoController = $logRetrocessoCalculoController;
        $this->eSocialService = $eSocialService;
    }
    
     public function calculoProgramacaoFerias (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $datasets = $request->getDataSet()->getRows()[0];
            
            foreach($datasets as $dataset){
                $dataSet = $dataset;
                
                if(!$dataSet['DTINIGOZOFERIAS']){
                    continue;
                }

                if(!$dataSet['DTINIGOZOINI']){
                    $dataSet['DTINIGOZOINI'] = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOFERIAS'],0, 10))->format('d/m/Y');
                }
                if(!$dataSet['DTINIGOZOFIM']){
                    $dataSet['DTINIGOZOFIM'] = DateUtil::getDataDeString(substr($dataSet['DTFIMGOZOFERIAS'],0, 10))->format('d/m/Y');
                }
                
                if(!$dataSet['DTPAGAMENTO']){
                    $dataSet['DTPAGAMENTO'] = DateUtil::subtraiIntervalo($dataSet['DTINIGOZOINI'], 2)->format('d/m/Y');
                }
                
                // Caso o processamento seja o retrocesso deve-se gravar o log da operação
                if($dataSet['TP_PROCESSAMENTO'] == "RETROCESSO") $this->logRetrocessoCalculoController->persistirLogRetrocesso($dataSet['TP_CALCULO'], $dataSet);
                
                $dtPag = DateUtil::getDataDeString($dataSet['DTPAGAMENTO']);
                $dtIniFerias = DateUtil::getDataDeString($dataSet['DTINIGOZOINI']);
                
                $dtmescompetenc = DateUtil::getDataDeString(substr($dataSet['DTPAGAMENTO'], 0, 10));
                $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes($dtmescompetenc);
                $dtmescompetenc = DateUtil::truncateData($dtmescompetenc);
                
                //$dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
                
                $this->PercorreFerias = new PercorreFerias($this->entityManager, $this->hcmParameters, $dataSet, $dtmescompetenc);
                $resultado = $this->PercorreFerias->percorre();
                
                if($dataSet['IDADIANT13SALAR'] == 'S' && $dtmescompetenc->format('m') != '11' && $dtmescompetenc->format('m') != '12'){
                    $dtmescompetenc = DateUtil::getDataDeString('01/'.substr($dataSet['DTINIGOZOINI'], 3, 7));
                    $dataset2 = (array) $dataSet;
                    $dataset2['TP_CALCULO'] = FpatipocalculoEntity::CALC_ADIANT_13SAL;
                    $this->PercorreDecimoTerceiro = new PercorreDecimoTerceiro($this->entityManager, $this->hcmParameters, $dataset2, $dtmescompetenc);
                    $resultado2 = $this->PercorreDecimoTerceiro->percorre(); 
                }
                /*
                if($dataSet['TP_PROCESSAMENTO'] == 'CALCULO' && array_key_exists('IDENVIAESOCIAL', $dataSet) && $dataSet['IDENVIAESOCIAL'] == 'S'){
                    $this->eSocialService->envioEsocialDescentralizadoCalculo($dataSet, $response);
                }*/
            }
            
            $message = null;

            if ($resultado['DEMONSTRATIVO'][0] && count($resultado['DEMONSTRATIVO'][0]['ARQUIVOMEMORIA']) > 0){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('memoriaCalculo', $resultado['DEMONSTRATIVO']));
            }
            
            if ($resultado === null) {
                throw new \Exception('Nenhum vinculo encontrado');
            }elseif(array_key_exists('CONFIRMACAO', $resultado)){
                $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' será realizado para  '.$resultado['QTDVINCULO'].' vínculos. Deseja continuar?', DTO\Response\Message::TYPE_WARNING);
            }elseif ($resultado['ABORT'] === true) {
                $message = ucwords(strtolower(self::$NM_PROCESSAMENTO[$dataSet['TP_PROCESSAMENTO']])).' Cálculo não finalizado, erros encontrados.<br>';
                foreach ($resultado['MENSAGENS'] as $mensagem) {
                    if ($mensagem['TIPO'] == \MsgCalculo::ERRO) {
                        $message .= strpos($message, $mensagem['MENSAGEM']) ? null : '<br> - ' . $mensagem['MENSAGEM'];
                    }
                }
                $message = new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_ERROR);
            }elseif ($resultado['ERRO'] === false) {
                $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
            } else {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'ERROSCALCULO'), array('nrvinculom' => 'ASC'));
                $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculo($resultado['CALCULO'], $this->hcmParameters->getNrorgTrabalho(), 'ERROSCALCULO');   
            }
            
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            } else {
                $row = array();
                foreach ($message as $mensagem) {
                    $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function calculo (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];

            if(!array_key_exists('NRORG',$dataSet) || is_null($dataSet['NRORG'])){
                $dataSet['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
            }
            
            if(array_key_exists('TP_CALCULO1',$dataSet)){
                $dataSet['TP_CALCULO'] = $dataSet['TP_CALCULO1']; 
            }
            
            // Caso o processamento seja o retrocesso deve-se gravar o log da operação
            if($dataSet['TP_PROCESSAMENTO'] == "RETROCESSO") $this->logRetrocessoCalculoController->persistirLogRetrocesso($dataSet['TP_CALCULO'], $dataSet);
            
            if($dataSet['DSORDEM'] === 'ESTRUT_ALFABETICA_TPVINC'){
                $dsinicio = array();
                $dsfim = array();
                if(array_key_exists('DSINICIO',$dataSet) && $dataSet['DSINICIO']){
                    $dsinicio['DSINICIO'] = $dataSet['DSINICIO'];
                }
                if(array_key_exists('DSINICIO2',$dataSet) && $dataSet['DSINICIO2']){
                    $dsinicio['DSINICIO2'] = $dataSet['DSINICIO2'];
                }
                if(array_key_exists('DSINICIO3',$dataSet) && $dataSet['DSINICIO3']){
                    $dsinicio['DSINICIO3'] = $dataSet['DSINICIO3'];
                }
                if(array_key_exists('DSFIM',$dataSet) && $dataSet['DSFIM']){                
                    $dsfim['DSFIM'] = $dataSet['DSFIM'];
                }
                if(array_key_exists('DSFIM2',$dataSet) && $dataSet['DSFIM2']){                
                    $dsfim['DSFIM2'] = $dataSet['DSFIM2'];
                }
                if(array_key_exists('DSFIM3',$dataSet) && $dataSet['DSFIM3']){
                    $dsfim['DSFIM3'] = $dataSet['DSFIM3'];
                }
                $dataSet['DSINICIO'] = $dsinicio;
                $dataSet['DSFIM'] = $dsfim;
            }
            
            if(empty($dataSet['DTPAGAMENTO']) && $dataSet['TP_PROCESSAMENTO'] == "RETROCESSO" && $dataSet['TP_CALCULO'] != FpatipocalculoEntity::CALC_FERIAS){
                $dataSet['DTPAGAMENTO'] = $this->hcmParameters->getDtCompetencia();
            }
            
            switch ($dataSet['TP_CALCULO']){
            case FpatipocalculoEntity::CALC_FOLHA:
                $this->PercorreFolha = new PercorreFolha($this->entityManager, $this->hcmParameters, $dataSet);
               
                $countVinc = $this->PercorreFolha->retornaQtVinculos();
                if($countVinc>200 && $dataSet['TP_PROCESSAMENTO'] != "RETROCESSO" && $dataSet['TP_PROCESSAMENTO'] != "SIMULACAO"){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('O numero de Vinculos ultrapassa a quantidade de 200.<br>Utilize a opção de Cálculo em lote', DTO\Response\Message::TYPE_ERROR));
                    return;
                }
                $DtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
                $dtPag = DateUtil::getDataDeString($dataSet['DTPAGAMENTO']);
                if($dtPag < $DtCompetencia){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A data de pagamento está anterior à competência atual do sistema.', DTO\Response\Message::TYPE_ERROR));	 
                    return;	  
                };
                $resultado = $this->PercorreFolha->percorre();
                
                break;
                
            case FpatipocalculoEntity::CALC_FOLHA_COMPLEMENTAR:
                $this->PercorreFolhaComplementar = new PercorreFolhaComplementar($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreFolhaComplementar->percorre();
                break;
            case FpatipocalculoEntity::CALC_FERIAS:
                
                $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
                
                /*if(!$dataSet['DTINIGOZOINI']){
                    $dataSet['DTINIGOZOINI'] = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOFERIAS'],0, 10));
                }
                if(!$dataSet['DTINIGOZOFIM']){
                    $dataSet['DTINIGOZOFIM'] = DateUtil::getDataDeString(substr($dataSet['DTFIMGOZOFERIAS'],0, 10));
                }*/
                
                #08/03/2023 Luan - Manutenção para o retrocesso funcionar sem nenhuma informação e retroceder de forma correta.
                if(empty($dataSet['DTPAGAMENTO']) && $dataSet['TP_PROCESSAMENTO'] == "RETROCESSO" && $dataSet['TP_CALCULO'] == FpatipocalculoEntity::CALC_FERIAS){
                    $dataSet['DTPAGAMENTO'] = $this->hcmParameters->getDtCompetencia();
                }
                
                if(!$dataSet['DTPAGAMENTO']){
                    $dataSet['DTPAGAMENTO'] = substr(DateUtil::subtraiIntervalo($dataSet['DTINIGOZOINI'], 2), 0, 10);
                }
                
                $validapgtFerias = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->hcmParameters->getNrorgTrabalho(), 0, $dtmescompetenc, 'NAO_VALIDA_DT_PG_FERIAS');                
                
                if($validapgtFerias != '1'){
                    if(substr($dataSet['DTPAGAMENTO'], 3, 7) != substr($this->hcmParameters->getDtCompetencia(), 3, 7)){	 
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Data de pagamento não esta dentro da competência do sistema', DTO\Response\Message::TYPE_ERROR));	 
                        return;	 
                    }
                }

                $dtPag = DateUtil::getDataDeString($dataSet['DTPAGAMENTO']);
                $dtIniFerias = DateUtil::getDataDeString($dataSet['DTINIGOZOINI']);
               /* if($dtPag > $dtIniFerias || $dtPag->format('d/m/Y') == $dtIniFerias->format('d/m/Y')){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Data de pagamento maior ou igual a data de início do gozo, Favor alterar', DTO\Response\Message::TYPE_ERROR));
                    return;
                }
                
                // faz com que a data de pagamento de férias seja usada como data de competência para o cálculo
                $dtmescompetenc = DateUtil::getDataDeString(substr($dataSet['DTPAGAMENTO'], 0, 10));
                $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes($dtmescompetenc);
                $dtmescompetenc = DateUtil::truncateData($dtmescompetenc);*/
                
                $this->PercorreFerias = new PercorreFerias($this->entityManager, $this->hcmParameters, $dataSet, $dtmescompetenc);
                $resultado = $this->PercorreFerias->percorre();
                
                if($dataSet['IDADIANT13SALAR'] == 'S' && $dtmescompetenc->format('m') != '11' && $dtmescompetenc->format('m') != '12'){
                    $dtmescompetenc = DateUtil::getDataDeString('01/'.substr($dataSet['DTINIGOZOINI'], 3, 7));
                    $dataset2 = clone $dataSet;
                    $dataset2['TP_CALCULO'] = FpatipocalculoEntity::CALC_ADIANT_13SAL;
                    $this->PercorreDecimoTerceiro = new PercorreDecimoTerceiro($this->entityManager, $this->hcmParameters, $dataset2, $dtmescompetenc);
                    $resultado2 = $this->PercorreDecimoTerceiro->percorre(); 
                }
                
                if($dataSet['TP_PROCESSAMENTO'] == 'CALCULO' && array_key_exists('IDENVIAESOCIAL', $dataSet) && $dataSet['IDENVIAESOCIAL'] == 'S'){
                    $this->eSocialService->envioEsocialDescentralizadoCalculo($dataSet, $response);
                }
                break;
            case FpatipocalculoEntity::CALC_13SAL:
                $this->PercorreDecimoTerceiro = new PercorreDecimoTerceiro($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreDecimoTerceiro->percorre();
                break;
            case FpatipocalculoEntity::CALC_ADIANT_13SAL:
                $dtPag = DateUtil::getDataDeString($dataSet['DTPAGAMENTO']);
                $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
                $dtIniFerias = DateUtil::getDataDeString($dataSet['DTINIGOZOINI']);
                
                if($dtPag < $dtmescompetenc){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A data de pagamento está anterior à competência atual do sistema.', DTO\Response\Message::TYPE_ERROR));	 
                    return;	 
                };
                $this->PercorreDecimoTerceiro = new PercorreDecimoTerceiro($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreDecimoTerceiro->percorre();
                if($resultado['msgCalculo']){
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $resultado['msgCalculo']));
                }
                break;
            case FpatipocalculoEntity::CALC_ADIANTAMENTO:
                $dtPag = DateUtil::getDataDeString($dataSet['DTPAGAMENTO']);
                $DtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
                if($dtPag < $DtCompetencia){	
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A data de pagamento está anterior à competência atual do sistema.', DTO\Response\Message::TYPE_ERROR));	 
                    return;	 
                };
                $this->PercorreAdiantamento = new PercorreAdiantamento($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreAdiantamento->percorre();
                break;
            case FpatipocalculoEntity::CALC_RESCISAO:
                $dtPag = DateUtil::getDataDeString($dataSet['DTPAGAMENTO']);
                $dtRes = DateUtil::getDataDeString($dataSet['DTRESCISAOINI']);
                $DtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());

                if($dtPag < $DtCompetencia){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A data de pagamento está anterior à competência atual do sistema.', DTO\Response\Message::TYPE_ERROR));	 
                    return;
                }

                $primeiroDiaDoMesDaRescisao = DateUtil::getPrimeiroDiaDoMes($dtRes);
                if($primeiroDiaDoMesDaRescisao  > $DtCompetencia){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A competência de cálculo está anterior à data de rescisão.', DTO\Response\Message::TYPE_ERROR));	 
                    return;
                }

                //Fecha mes anterior se for gravacao definitiva da rescisao.
                if($dataSet['TP_PROCESSAMENTO'] == 'CALCULO'){
                    $dtmescompetenc = DateUtil::subtraiIntervalo($this->hcmParameters->getDtCompetencia(), 1, DateUtil::MES);
                    $dataSet['TP_CALCULO1'] = 9;
                    $dataSet['TP_CALCULO'] = 9;
                    $this->PercorreFechamento = new PercorreFechamento($this->entityManager, $this->hcmParameters, $dataSet, $dtmescompetenc);
                    $resultado = $this->PercorreFechamento->percorre();
                }
                
                $dataSet['TP_CALCULO1'] = 6;
                $dataSet['TP_CALCULO'] = 6;
                $this->PercorreRescisao = new PercorreRescisao($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreRescisao->percorre();
                if($dataSet['TP_PROCESSAMENTO'] == 'CALCULO' && array_key_exists('IDENVIAESOCIAL', $dataSet) && $dataSet['IDENVIAESOCIAL'] == 'S'){
                    $this->eSocialService->envioEsocialDescentralizadoCalculo($dataSet, $response);
                }
                break;
            case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
                $this->PercorreRescisao = new PercorreRescisao($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreRescisao->percorre();
                if($dataSet['TP_PROCESSAMENTO'] == 'CALCULO'){
                    $this->eSocialService->verificaEnvioEsocialRescisaoComplem($dataSet, $response);
                }
                break;
            case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
                $this->PercorreProvisao = new PercorreProvisao($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreProvisao->percorre();
                break;
            case FpatipocalculoEntity::CALC_PROVISAO_OTIMIZADA:
                $this->PercorreProvisao = new PercorreProvisaoOtimizada($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreProvisao->percorre();
                break;
            case FpatipocalculoEntity::CALC_FECHAMENTO:
                $this->PercorreFechamento = new PercorreFechamento($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorreFechamento->percorre();
                break;
            case FpatipocalculoEntity::CALC_ADIANT_PLR:
                $message = new \Zeedhi\Framework\DTO\Response\Message('Indisponivel', DTO\Response\Message::TYPE_MESSAGE);
                $response->addMessage($message);
                return;
            case FpatipocalculoEntity::CALC_PLR:
                $this->PercorrePLR = new PercorrePLR($this->entityManager, $this->hcmParameters, $dataSet);
                $resultado = $this->PercorrePLR->percorre();
                break;
            default:
                throw new \Exception('Tipo de calculo '.$dataset['TP_CALCULO'].' invalido');
            }
            $message = null;

            if ($resultado['DEMONSTRATIVO'][0] && count($resultado['DEMONSTRATIVO'][0]['SIMULACAO'][0]) > 0){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('simulacao', $resultado['DEMONSTRATIVO']));
            }
            if ($resultado['DEMONSTRATIVO'][0] && count($resultado['DEMONSTRATIVO'][0]['ARQUIVOMEMORIA']) > 0){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('memoriaCalculo', $resultado['DEMONSTRATIVO']));
            }
            
            if ($resultado === null) {
                throw new \Exception('Nenhum vinculo encontrado');
            }elseif(array_key_exists('CONFIRMACAO', $resultado)){
                $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' será realizado para  '.$resultado['QTDVINCULO'].' vínculos. Deseja continuar?', DTO\Response\Message::TYPE_WARNING);
            }elseif ($resultado['ABORT'] === true) {
                $message = ucwords(strtolower(self::$NM_PROCESSAMENTO[$dataSet['TP_PROCESSAMENTO']])).' Cálculo não finalizado, erros encontrados.<br>';
                foreach ($resultado['MENSAGENS'] as $mensagem) {
                    if ($mensagem['TIPO'] == \MsgCalculo::ERRO) {
                        $message .= strpos($message, $mensagem['MENSAGEM']) ? null : '<br> - ' . $mensagem['MENSAGEM'];
                    }
                }
                $message = new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_ERROR);
            }elseif ($resultado['ERRO'] === false) {
                if($resultado['ALERTA']){
                    $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'ERROSCALCULO'), array('nrvinculom' => 'ASC'));
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado com Alertas.', DTO\Response\Message::TYPE_MESSAGE));
                    $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculo($resultado['CALCULO'], $this->hcmParameters->getNrorgTrabalho(), 'ERROSCALCULO');
                } else if($resultado['msgCalculo'][0]['IDTIPOMSG'] == 'Alerta'){
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $resultado['msgCalculo']));
                    $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado com Alertas.', DTO\Response\Message::TYPE_MESSAGE);
                } else {
                    $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
                }
            } else {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'ERROSCALCULO'), array('nrvinculom' => 'ASC'));
                $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculo($resultado['CALCULO'], $this->hcmParameters->getNrorgTrabalho(), 'ERROSCALCULO');   
            }
            
            
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            } else {
                $row = array();
                foreach ($message as $mensagem) {
                    $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function calculoApolo (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];
        
            if(array_key_exists('TP_CALCULO1',$dataSet)){
                $dataSet['TP_CALCULO'] = $dataSet['TP_CALCULO1']; 
            }
            if ($dataSet['DSORDEM'] === 'ESTRUT_ALFABETICA_TPVINC'){
                $dsinicio = array();
                $dsfim = array();
                if(array_key_exists('DSINICIO',$dataSet) && $dataSet['DSINICIO']){
                    $dsinicio['DSINICIO'] = is_array($dataSet['DSINICIO']) ? implode(',',$dataSet['DSINICIO']) : $dataSet['DSINICIO'];
                }
                if(array_key_exists('DSINICIO2',$dataSet) && $dataSet['DSINICIO2']){
                    $dsinicio['DSINICIO2'] = $dataSet['DSINICIO2'];
                }
                if(array_key_exists('DSINICIO3',$dataSet) && $dataSet['DSINICIO3']){
                    $dsinicio['DSINICIO3'] = is_array($dataSet['DSINICIO3']) ? implode(',',$dataSet['DSINICIO3']) : $dataSet['DSINICIO3'];
                }
                if(array_key_exists('DSFIM',$dataSet) && $dataSet['DSFIM']){                
                    $dsfim['DSFIM'] = is_array($dataSet['DSFIM']) ? implode(',',$dataSet['DSFIM']) : $dataSet['DSFIM'];
                }
                if(array_key_exists('DSFIM2',$dataSet) && $dataSet['DSFIM2']){                
                    $dsfim['DSFIM2'] = $dataSet['DSFIM2'];
                }
                if(array_key_exists('DSFIM3',$dataSet) && $dataSet['DSFIM3']){
                    $dsfim['DSFIM3']= $dataSet['FIM3']; 
                }
                $dataSet['DSINICIO'] = $dsinicio;
                $dataSet['DSFIM'] = $dsfim;
            }
            $percorreFilaApolo = new PreparaFilaApolo($this->entityManager, $this->hcmParameters, $dataSet);
            $resultado = $percorreFilaApolo->preparaFilaApolo();
            
            if ($resultado === null) {
                throw new \Exception('Nenhum vinculo encontrado');
            }
            
            $message = new \Zeedhi\Framework\DTO\Response\Message($resultado['MSG'], DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}