<?php

namespace HCM\FPA\ObrigacaoLegal\GRRF;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\FPA\Calculos\Geral\Estruturacao;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaGrpitemEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpagrrfmagnetEntity;
use HCM\FPA\Registro\grrf\Registro00GRRF;
use HCM\FPA\Registro\grrf\Registro90GRRF;
use HCM\FPA\Registro\RegistroGRRF;
use HCM\FPA\ObrigacaoLegal\GRRF\GRRF;

class PercorreGRRF{

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_cdOperador;
    private $_nrEstruturaTrab;
    private $_mesCompetencia;
    private $_nrTipoModalidadeCalculo;
    private $_tipoMovimento;
    private $_itemGrupo;
    private $_vinculom;
    private $_tipoHierarquia;
    private $_hierarquiaOrg;
    private $_calculoFolha;
    private $_grrfMagnetico;
    private $_registro00;
    private $_registro90;
    private $_pessoa;
    private $_estruturacao;
    private $_formulario;
    private $_nrRecolhimento;
    private $_idModalidade;
    private $_nomeInicial;
    private $_nomeFinal;
    private $_dtInicial;
    private $_dtFinal;
    private $_dtRecolhGRRF;
    private $_dsOrdem;
    private $_dsInicio;
    private $_dsFim;
    private $_nrTpHierarquiaOrg;
    private $_tpInscHierarq;
    private $_nrTpHierarqTomador;
    private $_tpInscHierarqTomador;
    private $_idProcessaVinculo;
    private $_idGeraMeioMagnetico;
    private $_filePath;
    private $_nrEstruturaResp;
    private $_tpInscResp;
    private $_nrPessoaResp;
    private $_idRecolhimentoFGTS;
    private $_dtFGTS;
    private $_idRecolhimentoGPS;
    private $_dtGPS;
    private $_nrGrpIncide;
    private $_listaNrCalculoFolha;
    private $_listaNrCalculoFolhaAnt;
    private $_retrocesso;
    private $_competencia13;
    private $_nrOcorreCal;
    private $_grrf;
    private $_listaTipoRegistro;
    private $_message;
    private $entityManager;
    private $hcmParameters;
    private $nomeArquivo;

    public function __construct($entityManager, $hcmParameters, $dataSet){

        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador = $hcmParameters->getCdOperador();
        $this->_nrEstruturaTrab = FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $this->_nrTipoModalidadeCalculo = $hcmParameters->getTipoModalidadeCalculo();
        $this->_tipoMovimento = FpatipomovimentEntity::TP_MOV_MENSAL;
        $this->_itemGrupo = new FpaGrpitemEntity($entityManager, $hcmParameters);
        $this->_vinculom = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_tipoHierarquia = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_hierarquiaOrg = new HierarquiaorgEntity($entityManager, $hcmParameters);
        $this->_calculoFolha = new FpacalculofolhaEntity($entityManager, $hcmParameters);
        $this->_grrfMagnetico = new FpagrrfmagnetEntity($entityManager, $hcmParameters);
        $this->_registro00 = new Registro00GRRF();
        $this->_registro90 = new Registro90GRRF();
        $this->_pessoa = new GpepessoaEntity($entityManager, $hcmParameters);
        $this->_estruturacao = new Estruturacao($entityManager, $hcmParameters);
        $this->_inicializaVariaveis($dataSet);

        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch (strtoupper($tp_processamento)){
            case FPA::PROC_COMPETENCIA_MENSAL :
                $this->_retrocesso = false;
                $this->_competencia13 = false;
                break;
            case FPA::PROC_COMPETENCIA_13 :
                $this->_retrocesso = false;
                $this->_competencia13 = true;
                break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true;
                $this->_competencia13 = false;
                break;
            default:
                throw new \Exception('Tipo de processo '.$tp_processamento.' invalido');

        }
        $this->_grrf  = new GRRF(
            $this->_nrOrg, $this->_nrOrgPadrao, $this->_cdOperador, $this->_mesCompetencia, $this->_grrfMagnetico, $dataSet,
            $this->_competencia13, $this->_itemGrupo, $this->_listaNrCalculoFolha, $entityManager, $hcmParameters, $this->_listaNrCalculoFolhaAnt
        );
    }

    public function _inicializaVariaveis($dataSet) {
        $this->nomeArquivo = "GRRF.RE";
        $this->_buscaDadosFormulario($dataSet);
        $this->_itemGrupo->carregaItensGrupoPorGrupoIncidencia($this->_nrOrg, $this->_nrOrgPadrao, $this->_nrGrpIncide);
        
        // Caso não tenha ocorrencia, define como 1
        if (!$this->_nrOcorreCal) {
            $this->_nrOcorreCal = 1;
        }
        
        // Lista de calculos do mês de geração da GRRF
        $this->_listaNrCalculoFolha = array();
        $arrayOcorrencias = explode(',',$this->_nrOcorreCal);
        foreach ($arrayOcorrencias as $nrocorrencia) {
            $calculo = $this->_calculoFolha->getCalculoFolha(
                $this->_nrOrg, $this->_tipoMovimento, $this->_nrTipoModalidadeCalculo, $nrocorrencia, $this->_mesCompetencia
            );
            if (is_object($calculo[0])) {
                $this->_listaNrCalculoFolha[] = $calculo[0]->getNrcalculofolha();
            }
        }
        if (empty($this->_listaNrCalculoFolha) || count($this->_listaNrCalculoFolha) <= 0) {
            throw new \Exception("Não foram encontrados cálculos para as ocorrências informadas.");
        }
        
        // Lista de cálculos de competências anteriores para informações complemetares caso os registros estejam marcados
        if ($this->_infCompl43 == 'S' || $this->_infCompl44 == 'S' || $this->_infCompl45 == 'S') {
            $this->_listaNrCalculoFolhaAnt = array();
            $arrayOcorrencias = explode(',',$this->_nrOcorreCal);
            foreach ($this->_infComplComps as $mescompetencia) {
                foreach ($arrayOcorrencias as $nrocorrencia) {
                    $dtmescompetencia = DateUtil::getDataDeString($mescompetencia,DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
                    $calculo = $this->_calculoFolha->getCalculoFolha(
                        $this->_nrOrg, $this->_tipoMovimento, $this->_nrTipoModalidadeCalculo, $nrocorrencia, $dtmescompetencia
                    );
                    if (is_object($calculo[0])) {
                        $this->_listaNrCalculoFolhaAnt[$dtmescompetencia->format('d/m/Y')][] = $calculo[0]->getNrcalculofolha();
                    }
                }
            }
            if (empty($this->_listaNrCalculoFolhaAnt) || count($this->_listaNrCalculoFolhaAnt) <= 0) {
                throw new \Exception("Não foram encontrados cálculos para as ocorrências informadas.");
            }
        }
        
    }

    private function _buscaDadosFormulario($dataSet){
        
        $this->_nrRecolhimento = $dataSet['NRRECOLHIMENTO'];
        $this->_idModalidade = $dataSet['IDMODALIDADE'];
        $this->_nomeInicial = $dataSet['NOMEINICIAL'];
        $this->_nomeFinal = $dataSet['NOMEFINAL'];
//        $this->_dtInicial = $this->_mesCompetencia;
//        $this->_dtFinal = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        $this->_dtInicial = $dataSet['DTINICIAL'];
        $this->_dtFinal = $dataSet['DTFINAL'];
        $this->_dtRecolhGRRF = $dataSet['DTRECOLHGRRF'];
        $this->_dsOrdem = $dataSet['DSORDEM'];
        $this->_dsInicio = $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_nrTpHierarquiaOrg = $dataSet['NRHIERARQORG'];
        $this->_tpInscHierarq = $dataSet['IDTPINSCHIERARQ'];
        $this->_nrTpHierarqTomador = $dataSet['NRHIERARTOMADOR'];
        $this->_tpInscHierarqTomador = $dataSet['IDTPINSCHIERTOM'];
        $this->_idProcessaVinculo = $dataSet['IDPROCVINCULO'];
        $this->_idGeraMeioMagnetico = $dataSet['IDGERAMEIOMAG'];
        $this->_filePath = $dataSet['DSCAMINHOARQUIVO'];
        $this->_nrEstruturaResp = $dataSet['NRESTRUTRESPONS'];
        $this->_tpInscResp = $dataSet['IDTPINSCRESPONS'];
        $this->_nrPessoaResp = $dataSet['NRPESSOARESPONS'];
        $this->_idRecolhimentoFGTS = $dataSet['IDRECOLHFGTS'];
        $this->_dtFGTS = $dataSet['DTRECOLHFGTS'];
        $this->_idRecolhimentoGPS = $dataSet['IDRECOLHGPS'];
        $this->_dtGPS = $dataSet['DTRECOLHGPS'];
        $this->_nrGrpIncide = $dataSet['NRGRPINCIDE'];
        $this->_nrOcorreCal = $dataSet['NROCORRECAL'];
        $this->_infCompl43 = $dataSet['IDINFOCOMPL43'];
        $this->_infCompl44 = $dataSet['IDINFOCOMPL44'];
        $this->_infCompl45 = $dataSet['IDINFOCOMPL45'];
        $this->_infComplComps = array_reverse($dataSet['DTMESCOMPETENCLIST']);
    }

    public function percorreVinculos(){
        $retorno = null;        
        if ($this->_retrocesso === false) {
            $path = FPA::getTempDirectory().'/'.strtoupper($this->nomeArquivo);
            if(file_exists($path)){
                unlink(realpath($path));
            }
            $teste = $this->_grrfMagnetico->inicializaListaRegistros(
                $this->_mesCompetencia, RegistroGRRF::LISTA_TIPO_REG_EMP, $this->_nrOrg, $this->_cdOperador
            );
            if ($this->_idProcessaVinculo == 'S') {
                $vinculos = $this->_vinculom->retornaVinculosDemitidosGeral(
                    $this->_nrOrg, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim, $this->_dtInicial,
                    $this->_dtFinal, $this->_dsOrdem, $this->_nrEstruturaTrab
                );
                if(!empty($vinculos)){
                    foreach ($vinculos as $vinculo) {
                        MsgCalculo::getInstance()->setDados($this->_listaNrCalculoFolha[0], $vinculo->getNrvinculom(), $this->_nrOrg, 'GRRF');
                        if ($vinculo->getNrvinculoempreg() !== false) {
                            try{
                                $this->_grrf->gerarGRRF($vinculo, $this->_tpInscHierarq, $this->_tpInscHierarqTomador);
                            }catch (\Exception $e){
                                MsgCalculo::getInstance()->adicionaMensagem($e->getMessage(),MsgCalculo::ERRO, true);
                            }
                        }else{
                            MsgCalculo::getInstance()->adicionaMensagem('Vínculo empregatício não informado para '.$vinculo->getNrvinculom().'-'.$vinculo->getNmvinculom());
                        }
                    }
                    $qtdRegistros = $this->_grrfMagnetico->countItems();
                    if ($qtdRegistros > 0) {
                        $this->_grrfMagnetico->persist();
                    }
                }else{
                    $retorno = 'Nenhum vínculo demitido foi encontrado para o mês de competência.';
                }
            }
            if( is_null($retorno) ){
                $this->finalizaMensagens();
    
                if ($this->_idGeraMeioMagnetico === 'S') {
                    $arquivo = $this->_geraMagnetico();
                }
                
                $retorno = array(
                    'CALCULO' => $this->_listaNrCalculoFolha[0],
                    'MENSAGENS' => ($this->_msg_idabort === true ? $this->_msg_mensagens : null),
                    'ERRO' => ($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO),
                    'ABORT' => $this->_msg_idabort,
                    'ARQUIVO' => $arquivo
                );
            }

            return $retorno;

        }else{
            $this->_grrf->retroceder();
            $this->limpaMensagens();
            return 'Retrocedido com Sucesso!';
        }
    }
    
    private function limpaMensagens(){
        $mensagens = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $this->_listaNrCalculoFolha[0], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'GRRF'));
        if (count($mensagens) > 0) {
            $this->entityManager->beginTransaction();
            foreach ($mensagens as $mensagem) {
                $this->entityManager->remove($mensagem);
                $this->entityManager->flush();
            }
            $this->entityManager->commit();
        }
    }

    private function _geraMagnetico(){

        $this->_preparaRegistro00();

        $this->_registro00->geraRegistro00();

        FPA::escreveArquivo($this->nomeArquivo, $this->_registro00->getLinhaRegistro().RegistroGRRF::QUEBRA_LINHA_CR_LF);
        
        $resultado = $this->_grrfMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg, $this->_cdOperador);
        foreach ($resultado as $item) {
            FPA::escreveArquivo($this->nomeArquivo, $item->getDslinharegistro().RegistroGRRF::QUEBRA_LINHA_CR_LF);
        }
        $this->_registro90->geraRegistro90();
        
        FPA::escreveArquivo($this->nomeArquivo, $this->_registro90->getLinhaRegistro());
        
        return $this->nomeArquivo;    
    }

    private function _preparaRegistro00(){

        $estrutura = $this->_estruturacao->retornaEstrutura($this->_nrEstruturaResp);

        if (is_object($estrutura) === true){

            $estrutura->endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
            $pessoa = $this->_pessoa->retornaPessoaAtual($this->_nrPessoaResp);

            if ($this->_tpInscResp === RegistroGRRF::CNPJ) {
                $this->_registro00->setTpInscricao(RegistroGRRF::CNPJ);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdcnpjestrut());
            } elseif ($this->_tpInscResp === RegistroGRRF::CEI) {
                $this->_registro00->setTpInscricao(RegistroGRRF::CEI);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdceiestrut());
            }

            $this->_registro00->setPessoaResp($pessoa);
            $this->_registro00->setEstruturaResp($estrutura);
            $this->_registro00->setCompetencia($this->_mesCompetencia);
            $this->_registro00->setCodigoRecolhimento($this->_nrRecolhimento);
            $this->_registro00->setModalidade($this->_idModalidade);
            $this->_registro00->setIdRecolhimentoFGTS($this->_idRecolhimentoFGTS);
            $this->_registro00->setDtFGTS($this->_dtFGTS);
            $this->_registro00->setDtRecolhGRRF($this->_dtRecolhGRRF);
            $this->_registro00->setIdRecolhimentoGPS($this->_idRecolhimentoGPS);
            $this->_registro00->setDtGPS($this->_dtGPS);
            $this->_registro00->setCompetencia13($this->_competencia13);
        }else{
            MsgCalculo::getInstance()->adicionaMensagem('Estrutura não encontrada.');
        }
    }
    
    private function finalizaMensagens() {
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ERRO;
        } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $this->_msg_idstatus != MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ALERTA;
        }
        MsgCalculo::getInstance()->salvaMensagens();
        MsgCalculo::getInstance()->clear();
        if ($msgcalculo['IDABORT'] === true) {
            $this->_msg_idabort = true;
            $this->_msg_mensagens = $msgcalculo['MENSAGENS'];
            return false;    
        } 
        return true;
    }
}