<?php

namespace HCM\Geral\Controllers;

use HCM\Util\Repositories;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\EstruturasService;
use Zeedhi\Framework\DTO\Response\Error;
use HCM\Geral\Controllers\ParceiroNegocioController;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\Util;
use HCM\Util\FPA;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\CrudService;
use HCM\Geral\Controllers\HierarquiaGridController;
use HCM\Util\DateUtil;
use HCM\Util\ConsultaInscricaoRF;

class EstruturaController extends Crud {
    
    protected $dataSourceName = 'estrutura';
    protected $dataSourceName_get = 'getEstrutura';
    protected $dataSourceName_getAtivas = 'getEstruturasAtivas';
    protected $dataSourceName_getGerLeg = 'getEstruturaGerLeg';
    protected $dataSourceName_getCustom = 'estruturaCustom_get';
    protected $dataSourceName_getSindPadr = 'getEstruturaSindicatoPadrao';
    protected $parameterBagColumns = array('NRORG' , 'P_NRORG', 'P_DTMESCOMPETENC', 'DTMESCOMPETENC','NRORG_PADRAO', 'GERENCIAL', 'LEGAL', 'P_NRORG_PADRAO', 'P_PADRAOSIND', 'IDATIVO', 'P_NRESTRUTLEGALOPER', 'P_NRESTRUTGERENOPER');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EstruturasService $estruturasService, CrudService $crudService, ParceiroNegocioController $parceiroNegocioController, EntityManager $entityManager, HierarquiaGridController $hierarquiaGridController, ConsultaInscricaoRF $consultaInscricaoRf){
        $this->parameterBag = $parameterBag;
        $this->estruturasService = $estruturasService;
        $this->hcmParameters   = $hcmParameters;
		$this->crudService = $crudService;
		$this->parceiroNegocioController = $parceiroNegocioController;
        $this->entityManager = $entityManager;
        $this->hierarquiaGridController = $hierarquiaGridController;
        $this->consultaInscricaoRf = $consultaInscricaoRf;
        parent::__construct($dataSourceManager);
    }
    
    public function getEstrutura(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        foreach ($filterData->getConditions() as $condition) {
            if($condition['columnName'] === 'LEGAL'){
                $filterData->addCondition('NRTIPOESTRUTURA', $this->hcmParameters->getTipoEstruturaLegalOrg());
            }
            else if($condition['columnName'] === 'GERENCIAL'){
                $filterData->addCondition('NRTIPOESTRUTURA', $this->hcmParameters->getTipoEstruturaGerencialOrg());
            }
        }
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet); 
    }
    
    public function getEstruturasAtivas(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_getAtivas);
        foreach ($filterData->getConditions() as $condition) {
            if($condition['columnName'] === 'LEGAL'){
                $filterData->addCondition('NRTIPOESTRUTURA', $this->hcmParameters->getTipoEstruturaLegalOrg());
            }
            else if($condition['columnName'] === 'GERENCIAL'){
                $filterData->addCondition('NRTIPOESTRUTURA', $this->hcmParameters->getTipoEstruturaGerencialOrg());
            }
        }
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet); 
    }
    
    public function getEstruturaSuperior(DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $parameters = array();
        foreach ($filters as $position => $filter) {
            unset($filter['operator']);
            foreach ($filter as $key => $value) {
                if ($key == 'columnName') {
                    $parameters[$value] = $filter['value'];
                }
            }
        }
        // $parameters['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
        // $parameters['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
        if(array_key_exists('NRTIPOESTRUTURA', $parameters)){
            $parameters['NRTIPOESTRUTURA'] = is_array($parameters['NRTIPOESTRUTURA']) ? implode($parameters['NRTIPOESTRUTURA'], ','): $parameters['NRTIPOESTRUTURA'];
        }else{
            $parameters['NRTIPOESTRUTURA'] = null;
        }
        Util::getValueInFilterDataSourceByName($filters, array('NRORG', 'DTMESCOMPETENC', 'NRTIPOESTRUTURA'), true);
        $conditions = Util::createQueryFiltered($filters);
        $estruturas = $this->estruturasService->getEstruturaSuperior($parameters, $conditions);
        $estruturas = is_array($estruturas) ? $estruturas : array();
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getEstruturaSuperior', $estruturas)); 
    }
    
    public function getEstruturaCustom(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_getCustom);
        $temIdAtivo = false;
        foreach ($filterData->getConditions() as $condition) {
            if($condition['columnName'] === 'LEGAL'){
                $filterData->addCondition('NRTIPOESTRUTURA', $this->hcmParameters->getTipoEstruturaLegalOrg());
            }
            else if($condition['columnName'] === 'GERENCIAL'){
                $filterData->addCondition('NRTIPOESTRUTURA', $this->hcmParameters->getTipoEstruturaGerencialOrg());
            }
            else if($condition['columnName'] === 'IDATIVO')
                $temIdAtivo = true;
        }
        if(!$temIdAtivo) $filterData->addCondition('IDATIVO', 'S');
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet); 
    }
    
    public function getEstruturaSindicatoPadrao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_getSindPadr);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet); 
    }
    
    public function deleteEstrutura(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->estruturasService->deleteEstrutura($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch (\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }else{
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }  
        }
    }
    
    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $conditions = array();
        $conditions[] = [
            'columnName' => 'NRORG',
            'value' => $this->hcmParameters->getNrorgTrabalho(),
            'operator' => '='
        ];
        $conditions[] = [
            'columnName' => 'DTMESCOMPETENC',
            'value' => $this->hcmParameters->getDtCompetencia(),
            'operator' => '='
        ];
        
        if($request->getRoutePath() == '/estrutura_legal'){
            $conditions[] = ['columnName' => 'NRTIPOESTRUTURA',
                             'value'      => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                             'operator'   => '='];
            
            $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutlegalOper());                             
                             
        } else if($request->getRoutePath() == '/estrutura_gerencial'){
            $conditions[] = ['columnName' => 'NRTIPOESTRUTURA',
                             'value'      => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
                             'operator'   => '='];
            
            $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutgerenOper());
            
        } else if($request->getRoutePath() == '/getEstruturaGerLeg'){
            $filterData->setDataSourceName($this->dataSourceName_getGerLeg);
            
            array_push($this->parameterBagColumns, 'NRTPESTRUTLEGAL');
            $conditions[] = ['columnName' => 'NRTPESTRUTLEGAL',
                             'value'      => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                             'operator'   => '='];

            $conditions[] = ['columnName' => 'NRTIPOESTRUTURA',
                             'value'      => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
                             'operator'   => '='];

            $conditions[] = ['columnName' => 'NRORG_PADRAO',
                             'value'      => $this->hcmParameters->getNrorgPadrao(),
                             'operator'   => '='];

        } else if ($request->getRoutePath() == '/estrutura_sindical'){
            $conditions[] = ['columnName' => 'NRTIPOESTRUTURA',
                             'value'      => 10,
                             'operator'   => '='];
                             
        } else if ($request->getRoutePath() == '/estrutura'){
            foreach ($filterData->getConditions() as $row) {
                $conditions[] = $row;
            }
        }
        
        
        foreach ($filterData->getConditions() as $filter) {
            $conditions[] = $filter;
        }
        $filterDataNew = new FilterCriteria(
            $filterData->getDataSourceName(),
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );
        $newFilterData = $this->populateParameterBag($filterDataNew);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getEstruturas($request, $response) {

        try {
            $filters = \HCM\Util\Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());            
            $parameters = array();
            foreach ($filters as $key1=>$filter) {
                unset($filter['operator']);
                foreach ($filter as $key2 => $value) {
                    if($key2 == 'value' && $value == '%%'){
                        unset($filters[$key1]);
                    }else if ($key2 == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG','NRTIPOESTRUTURA','P_NRORG_PADRAO'), true);
            $conditions = Util::createQueryFiltered($filters);
            
            $result = $this->estruturasService->getEstruturas($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getEstruturas', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
    
    public function getEstruturaLegalGerencial($request, $response) {

        try {
            //$filters = \HCM\Util\Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());            
            /*foreach ($filters as $key1=>$filter) {
                unset($filter['operator']);
                foreach ($filter as $key2 => $value) {
                    if($key2 == 'value' && $value == '%%'){
                        unset($filters[$key1]);
                    }else if ($key2 == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array(), true);
            $conditions = Util::createQueryFiltered($filters);*/
            $conditions = null;
            
            $result = $this->estruturasService->getEstruturaLegalGerencial($conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getEstruturaLegalGerencial', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getLocalidadeEstrutura($request, $response) {

        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }

            $result = $this->estruturasService->getLocalidadeEstrutura($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getLocalidadeEstrutura', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setMultiplasEstruturas($request, $response){
        try{
    	    foreach ($request->getDataSet()->getRows() as &$row) {
                if($row['TIPO_MULTIPLAS_ESTRUTURAS'] == 'TOMADOR/GERENCIAL'){
                    
                    $nrEstrutSuperior = $nrEstrutRelacionada = null;
                    
                    // Persistir a estrutura do tomador
                    $nrTpEstrutSuperior = $row['ETRUTURA_TOMADOR']['NRTIPOESTRUTURA'];
                    $newRow = new \Zeedhi\Framework\DTO\Row($row['ETRUTURA_TOMADOR']);
                    $newDataSet = new \Zeedhi\Framework\DataSource\DataSet("", [$newRow]);
                    $requestSuperior = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, null, null, null);
                    $responseSuperior = clone $response;
                    
                    $this->setEstrutura($requestSuperior, $responseSuperior);
                    foreach($responseSuperior->getDataSets() as $dataSet){
                        foreach($dataSet->getRows() as $data){
                            if(isset($data['NRESTRUTURAM']) && !empty($data['NRESTRUTURAM'])) $nrEstrutSuperior = $data['NRESTRUTURAM']; 
                        }
                    } 

                    // Persistir a estrutura gerencial
                    $nrTpEstrutRel = $row['ESTRUTURA_GERENCIAL']['NRTIPOESTRUTURA'];
                    $newRow = new \Zeedhi\Framework\DTO\Row($row['ESTRUTURA_GERENCIAL']);
                    $newDataSet = new \Zeedhi\Framework\DataSource\DataSet("", [$newRow]);
                    $requestRelacionada = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, null, null, null);
                    $responseRelacionada = clone $response;
                    $this->setEstrutura($requestRelacionada, $responseRelacionada);
                    foreach($responseRelacionada->getDataSets() as $dataSet){
                        foreach($dataSet->getRows() as $data){
                            if(isset($data['NRESTRUTURAM']) && !empty($data['NRESTRUTURAM'])) $nrEstrutRelacionada = $data['NRESTRUTURAM']; 
                        }
                    } 

                    
                    // Persistir a hierarquia TOMADOR/GERENCIAL
                    $tpHierarquia = $this->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findOneBy( 
                        [ 
                         'nrtipoestrutsup' => $nrTpEstrutSuperior,
                         'nrtipoestrutrel' => $nrTpEstrutRel,
                         'nrorg'           => [$this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao()],
                        ]
                    );
                    if($tpHierarquia){
                        $rowHierarquia = $this->validaCamposHierarquia($tpHierarquia->getNrtipohierarqorg(), $nrEstrutSuperior, $nrEstrutRelacionada);
                        $newRow = new \Zeedhi\Framework\DTO\Row($rowHierarquia);
                        $newDataSet = new \Zeedhi\Framework\DataSource\DataSet("", [$newRow]);
                        $requestHierarquia = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, null, null, null);
                        $responseHierarquia = clone $response;
                        $this->hierarquiaGridController->setHierarquiaGrid($requestHierarquia, $responseHierarquia);
                    }
                    
                    $response->addDataset(new DataSet('sucesso', ["SUCESSO"]));
                }
                
                else if($row['TIPO_MULTIPLAS_ESTRUTURAS'] == 'LEGAL/GERENCIAIS'){
                    
                    $nrEstrutSuperior = null;

                    // Persistir a estrutura legal
                    $nrTpEstrutSuperior = $row['ESTRUTURA_LEGAL']['NRTIPOESTRUTURA'];
                    $newRow = new \Zeedhi\Framework\DTO\Row($row['ESTRUTURA_LEGAL']);
                    $newDataSet = new \Zeedhi\Framework\DataSource\DataSet("", [$newRow]);
                    $requestSuperior = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, null, null, null);
                    $responseSuperior = clone $response;
                    $this->setEstrutura($requestSuperior, $responseSuperior);
                    foreach($responseSuperior->getDataSets() as $dataSet){
                        foreach($dataSet->getRows() as $data){
                            if(isset($data['NRESTRUTURAM']) && !empty($data['NRESTRUTURAM'])) $nrEstrutSuperior = $data['NRESTRUTURAM']; 
                        }
                    } 

                    $nrTpEstrutRel = $row['ETRUTURAS_GERENCIAIS']['NRTIPOESTRUTURA'];

                    // Persistir a hierarquia TOMADOR/GERENCIAL
                    $tpHierarquia = $this->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findOneBy( 
                        [ 
                         'nrtipoestrutsup' => $nrTpEstrutSuperior,
                         'nrtipoestrutrel' => $nrTpEstrutRel,
                         'nrorg'           => [$this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao()],
                        ]
                    );
                    if($tpHierarquia){
                        foreach($row['ETRUTURAS_GERENCIAIS']['NRESTRUTGEREN'] as $key=>$nrEstrutGeren){
                            $rowHierarquia = $this->validaCamposHierarquia($tpHierarquia->getNrtipohierarqorg(), $nrEstrutSuperior, $nrEstrutGeren);
                            $newRow = new \Zeedhi\Framework\DTO\Row($rowHierarquia);
                            $newDataSet = new \Zeedhi\Framework\DataSource\DataSet("", [$newRow]);
                            $requestHierarquia = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, null, null, null);
                            $responseHierarquia = clone $response;
                            $this->hierarquiaGridController->setHierarquiaGrid($requestHierarquia, $responseHierarquia);
                        }
                    }
                    
                    $response->addDataset(new DataSet('sucesso', ["SUCESSO"]));
                }
                else if($row['TIPO_MULTIPLAS_ESTRUTURAS'] == 'TOMADOR/GERENCIAIS'){
                    
                    $nrEstrutSuperior = null;

                    // Persistir a estrutura tomador
                    $nrTpEstrutSuperior = $row['ESTRUTURA_TOMADOR']['NRTIPOESTRUTURA'];
                    $newRow = new \Zeedhi\Framework\DTO\Row($row['ESTRUTURA_TOMADOR']);
                    $newDataSet = new \Zeedhi\Framework\DataSource\DataSet("", [$newRow]);
                    $requestSuperior = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, null, null, null);
                    $responseSuperior = clone $response;
                    $this->setEstrutura($requestSuperior, $responseSuperior);
                    foreach($responseSuperior->getDataSets() as $dataSet){
                        foreach($dataSet->getRows() as $data){
                            if(isset($data['NRESTRUTURAM']) && !empty($data['NRESTRUTURAM'])) $nrEstrutSuperior = $data['NRESTRUTURAM']; 
                        }
                    } 

                    $nrTpEstrutRel = $row['ETRUTURAS_GERENCIAIS']['NRTIPOESTRUTURA'];

                    // Persistir a hierarquia TOMADOR/GERENCIAL
                    $tpHierarquia = $this->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findOneBy( 
                        [ 
                         'nrtipoestrutsup' => $nrTpEstrutSuperior,
                         'nrtipoestrutrel' => $nrTpEstrutRel,
                         'nrorg'           => [$this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao()],
                        ]
                    );
                    if($tpHierarquia){
                        foreach($row['ETRUTURAS_GERENCIAIS']['NRESTRUTGEREN'] as $key=>$nrEstrutGeren){
                            $rowHierarquia = $this->validaCamposHierarquia($tpHierarquia->getNrtipohierarqorg(), $nrEstrutSuperior, $nrEstrutGeren);
                            $newRow = new \Zeedhi\Framework\DTO\Row($rowHierarquia);
                            $newDataSet = new \Zeedhi\Framework\DataSource\DataSet("", [$newRow]);
                            $requestHierarquia = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, null, null, null);
                            $responseHierarquia = clone $response;
                            $this->hierarquiaGridController->setHierarquiaGrid($requestHierarquia, $responseHierarquia);
                        }
                    }
                    
                    $response->addDataset(new DataSet('sucesso', ["SUCESSO"]));
                }

		    }
        } catch (\Exception $e) {
			$response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
		}
    }


    public function setEstrutura($request, $response) {
		try {
		    $envios = null;
		    $mensagem = null;
		    
		    foreach ($request->getDataSet()->getRows() as &$row) $this->validarCampos($row);
		    $row['IDVERIFICAVIGENCIAESO'] = array_key_exists('IDVERIFICAVIGENCIAESO', $row) ? $row['IDVERIFICAVIGENCIAESO'] : 'N';
            if($row['IDVERIFICAVIGENCIAESO'] == 'S'){
                $envios = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosPorNomeEventoChave($row['NRORG'], $row['NMEVENTO'], $row['NRCHAVE'], 1);   
            }
		    if(empty($envios) ||  $row['IDVERIFICAVIGENCIAESO'] == 'N'){
		        
    		    //Salva o parceiro de negocio e retorna o NRPARCNEGOCIO para ser salvo na tabela de estrutura
    		    $this->parceiroNegocioController->setParcNegocio($request, $response);
    
    		    if(!empty($response->getDataSets())){
    		        
    		        foreach ($request->getDataSet()->getRows() as &$row) $row["NRPARCNEGOCIO"] = $response->getDataSets()[0]->getRows()[0]["NRPARCNEGOCIO"];
            	    $this->dataSourceName = 'estruturam';
            	    $this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
            	    $this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            	    $rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
            	    $this->crudService->save($this->dataSourceName, $rows);
            	    $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            			
            		$currentRow = current($request->getDataSet()->getRows());
            		
            		if(array_key_exists('CDCNPJESTRUT', $currentRow)){
            		    $currentRow['CDCNPJESTRUT'] = preg_replace('/[^\d-]/i', '', $currentRow['CDCNPJESTRUT']);
            		}
            		if(array_key_exists('NMAGENCIA', $currentRow)){
            		    $currentRow['NMAGENCIA'] = preg_replace('.', '', $currentRow['NMAGENCIA']);
            		    $currentRow['NMAGENCIA'] = preg_replace('-', '', $currentRow['NMAGENCIA']);
            		}
            		if(array_key_exists('CDCONTCORR', $currentRow)){
            		    $currentRow['CDCONTCORR'] = preg_replace('.', '', $currentRow['CDCONTCORR']);
            		    $currentRow['CDCONTCORR'] = preg_replace('-', '', $currentRow['CDCONTCORR']);
            		}
            		

            		if(isset($currentRow['saveHistoryTable'])) {
                        
                        if (!$currentRow['__is_new']) {
                            if (!$this->checkHistory($response, $currentRow)) {
                                // is new history
                                $currentRow['__is_new'] = true;
                            }
                        }
                        
                        if( $currentRow['__is_new'] ){
                            if( $row['NRESTRUTURAM'] ){
                		        $nrorg = $row['NRORG'];
                    		    $nrestruturam = $row['NRESTRUTURAM'];
                    		    $competencia = $row['DTMESCOMPETENC'];
                    		    $competencia = DateUtil::getDataDeString( substr($competencia, 0, 10) );
                    		    $competencia = DateUtil::truncateData( $competencia );
                    		    
                    		    $estrutura = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->findOneBy(array('nrestruturam' => $nrestruturam, 'nrorg' => $nrorg, 'dtmescompetenc' => $competencia ));
                		        
                		        if( $estrutura ){
                		            $mensagem = "Operação bloqueada! Favor atualizar os dados da tela antes de prosseguir com a respectiva atualização.";
                		        }
            		        }
                        }
                        
                        if( !$mensagem ){
                			$tableMasterPK = strtoupper($currentRow['saveHistoryTable']['__table_master_pk']);
                			$newCode = array_filter($this->crudService->getColumnsToReturn(), function ($element) use ($tableMasterPK) {
                				return $element[$tableMasterPK];
                			});
                			
                			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($currentRow['saveHistoryTable']['__table_history_name']);
                			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
                			$currentRow[strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])] = intval($newCode[0][strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])]);
            
                            // set history properties
                            $currentRow['__new_code_pk'] = $currentRow['saveHistoryTable']['__table_history_name'];
                            $currentRow['__table_pk'] = $currentRow['saveHistoryTable']['__table_history_pk'];
            
                	       	$newDataSet = new DataSet($this->dataSourceName, array($currentRow));
                	       	$newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                			$rows = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
                			
                			$this->crudService->save($this->dataSourceName, $rows);
                			
                			$response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
                        } else {
                            $response->addDataset(new DataSet('operacaoBloqueada', ['MENSAGEM' => $mensagem]));
                        }
                    }
    			}
		    } else {
		        $mensagem = 'Registro possui o(s) evento(s) S-1005,S-1020 já transmitido para o eSocial com a data de vigência atual. Ao alterar a vigência o mesmo ficará incompatível com o sistema único. Deseja prosseguir com a alteração?';
		        $response->addDataset(new DataSet('verificaVigenciaEnvio', ['MENSAGEM' => $mensagem]));
		    }
		} catch (\Exception $e) {
		    $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
		}
	}
    
    private function validarCampos(&$row){
        
        if(isset($row['CDCNPJESTRUT']) && !empty($row['CDCNPJESTRUT'])) $row['CDCNPJESTRUT'] = preg_replace('/[\.\/-]/', '', $row['CDCNPJESTRUT']);
        
        $row['CDTIPOPARCPRINCIPAL'] = $row['NRTIPOESTRUTURA'] == 26 ? 'FORNECEDOR' : 'ESTRUTURA';
        $row['DTNASCIFUNDPARC'] = $row['DTINIVIGENCIA'];
        $row['CDTIPOINSCRICAO'] = $row['CDTIPOINSCRICAO'];
        $row['NRINSCRICAOPARC'] = $row['CDCNPJESTRUT'];
        $row['IDPESSOAFISICA'] = 'N';
        $row['IDINSTITUICAO'] = 'N';        
        $row['IDPARCFUNDIDO'] = 'N';
        
        $cnpj = isset($row['CDCNPJESTRUT']) && !empty($row['CDCNPJESTRUT']) ? $row['CDCNPJESTRUT'] : null;
        $row['CODCONVENIO'] = isset($row['CODCONVENIO']) && !empty($row['CODCONVENIO']) ? $row['CODCONVENIO'] : null;
        
        if(strlen($cnpj) === 14) $row['CDTIPOINSCRICAO'] = 'CNPJ';
        else if(strlen($cnpj) === 11) $row['CDTIPOINSCRICAO'] = 'CPF';
        else $row['CDTIPOINSCRICAO'] = 'LIVRE';
        
        
        if( !isset($row['NMRAZSOCESTRUT']) || empty($row['NMRAZSOCESTRUT']) ) $row['NMRAZSOCESTRUT'] = $row['NMFANTASIA'];
        
        if( !isset($row['NMESTRUTURAH']) || empty($row['NMESTRUTURAH']) ) $row['NMESTRUTURAH'] = $row['NMFANTASIA'];

        $row['NMESTRUTURAM'] = $row['NMESTRUTURAH'];
        $row['NMPRINCIPALPARC'] = isset($row['NMRAZSOCESTRUT']) && !empty($row['NMRAZSOCESTRUT']) ? $row['NMRAZSOCESTRUT'] : $row['NMFANTASIA'];
        $row['NMSECUNDARIPARC'] = $row['NMFANTASIA'];
        $row['IDATIVO'] = isset($row['IDATIVO']) && !empty($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
        $row['NRORG'] = isset($row['NRORG']) && !empty($row['NRORG']) ? $row['NRORG'] : $this->hcmParameters->getNrorgTrabalho();
        $row['NRTPEVENTOCTBL'] = isset($row['NRTPEVENTOCTBL']) && !empty($row['NRTPEVENTOCTBL']) ? $row['NRTPEVENTOCTBL'] : null;
        $row['DTESOCIAL'] = isset($row['DTESOCIAL']) && !empty($row['DTESOCIAL']) ? $row['DTESOCIAL'] : null;
        $row['CDINTESTRUTEXPFIL'] = isset($row['CDINTESTRUTEXPFIL']) && !empty($row['CDINTESTRUTEXPFIL']) ? $row['CDINTESTRUTEXPFIL'] : null;
        $row['CDINTESTRUTEXP'] = isset($row['CDINTESTRUTEXP']) && !empty($row['CDINTESTRUTEXP']) ? $row['CDINTESTRUTEXP'] : null;
        $row['CDEXPPDRATEIO'] = isset($row['CDEXPPDRATEIO']) && !empty($row['CDEXPPDRATEIO']) ? $row['CDEXPPDRATEIO'] : null;
        $row['NRREGIMEAPUR'] = isset($row['NRREGIMEAPUR']) && !empty($row['NRREGIMEAPUR']) ? $row['NRREGIMEAPUR'] : null;
        $row['NRGRUPOESOCIAL'] = isset($row['NRGRUPOESOCIAL']) && !empty($row['NRGRUPOESOCIAL']) ? $row['NRGRUPOESOCIAL'] : null;
        $row['NRPARCREGAMB'] = isset($row['NRPARCREGAMB']) && !empty($row['NRPARCREGAMB']) ? $row['NRPARCREGAMB'] : null;
        $row['NRPARCMONBIO'] = isset($row['NRPARCMONBIO']) && !empty($row['NRPARCMONBIO']) ? $row['NRPARCMONBIO'] : null;
        $row['DTINIRESAMB'] = isset($row['DTINIRESAMB']) && !empty($row['DTINIRESAMB']) ? $row['DTINIRESAMB'] : null;
        $row['DTFIMRESAMB'] = isset($row['DTFIMRESAMB']) && !empty($row['DTFIMRESAMB']) ? $row['DTFIMRESAMB'] : null;
        $row['NRTPOBRIGAPRENDIZ'] = isset($row['NRTPOBRIGAPRENDIZ']) ? $row['NRTPOBRIGAPRENDIZ'] : null;
        $row['NRTPOBRIGAPCD'] = isset($row['NRTPOBRIGAPCD']) ? $row['NRTPOBRIGAPCD'] : null;
        $row['CDCAEPFESTRUT'] = isset($row['CDCAEPFESTRUT']) && !empty($row['CDCAEPFESTRUT']) ? $row['CDCAEPFESTRUT'] : null;
        $row['NRESTRUTURAM'] = isset($row['NRESTRUTURAM']) && !empty($row['NRESTRUTURAM']) ? $row['NRESTRUTURAM'] : null;
        $row['NRESTRUTURAH'] = isset($row['NRESTRUTURAH']) && !empty($row['NRESTRUTURAH']) ? $row['NRESTRUTURAH'] : null;
        $row['CODCONVENIO'] = isset($row['CODCONVENIO']) && !empty($row['CODCONVENIO']) ? $row['CODCONVENIO'] : null;
        $row['DTMESCOMPETENC'] = isset($row['DTMESCOMPETENC']) && !empty($row['DTMESCOMPETENC']) ? $row['DTMESCOMPETENC'] : $this->hcmParameters->getDtCompetencia();
        $row['DTINCLUSAO'] = isset($row['DTINCLUSAO']) && !empty($row['DTINCLUSAO']) ? $row['DTINCLUSAO'] : $this->hcmParameters->getDataAtual(false);
        $row['DTULTATU'] = isset($row['DTULTATU']) && !empty($row['DTULTATU']) ? $row['DTULTATU'] : $this->hcmParameters->getDataAtual(false);
        $row['NRORGINCLUSAO'] = isset($row['NRORGINCLUSAO']) && !empty($row['NRORGINCLUSAO']) ? $row['NRORGINCLUSAO'] : $this->hcmParameters->getNrorgTrabalho();
        $row['NRORGULTATU'] = isset($row['NRORGULTATU']) && !empty($row['NRORGULTATU']) ? $row['NRORGULTATU'] : $this->hcmParameters->getNrorgTrabalho();
        $row['CDOPERINCLUSAO'] = isset($row['CDOPERINCLUSAO']) && !empty($row['CDOPERINCLUSAO']) ? $row['CDOPERINCLUSAO'] : $this->hcmParameters->getCdOperador();
        $row['CDOPERULTATU'] = isset($row['CDOPERULTATU']) && !empty($row['CDOPERULTATU']) ? $row['CDOPERULTATU'] : $this->hcmParameters->getCdOperador();
    }
    
    public function validaCamposHierarquia($tipoHierarquia, $nrEstrutSup, $nrEstrutRel){
        return [
                "NRORG" => $this->hcmParameters->getNrorgTrabalho(),                  
                "NRTIPOHIERARQORG" => $tipoHierarquia,       
                "NMHIERARQUIAORG" => null,  
                "NRESTRUTURAMSUP" => $nrEstrutSup,            
                "NRESTRUTURAMREL" => $nrEstrutRel, 
                "DTINIVIGENCIA" => $this->hcmParameters->getDtCompetencia(),   
                "DTFIMVIGENCIA" => null,                      
                "DTINCLUSAO" => $this->hcmParameters->getDataAtual(false),                
                "DTULTATU" => $this->hcmParameters->getDataAtual(false),                          
                "NRORGINCLUSAO" => $this->hcmParameters->getNrorgTrabalho(),                              
                "CDOPERINCLUSAO" => $this->hcmParameters->getCdoperador(),           
                "NRORGULTATU" => $this->hcmParameters->getNrorgTrabalho(),                                 
                "CDOPERULTATU" => $this->hcmParameters->getCdoperador(),             
                "IDATIVO" => 'S',                  
                "__is_new" => true,                   
            ];
    }
    
    
    public function checkHistory($response, $row) {
        $condition = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->hcmParameters->getNrorgTrabalho()
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_master_pk']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_master_pk'])]
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_competenc']),
                "operator" => "=",
                "value" => $this->hcmParameters->getDtCompetencia()
            )
        );
        
        $filterCriteria = new FilterCriteria($row['saveHistoryTable']['__table_history_name'], $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array());
       
        $this->crudService->find($response, $newFilterCriteria);
        if (empty($response->getDataSets()[1]->getRows())) 
            return false;
        else 
            return true;
        
    }
    
    public function getEntidadeEducacional($request, $response){
        try{
            $conditions = array();
            $row = $request->getFilterCriteria()->getConditions();
            foreach($row as $key => $condition){
                $conditions[$condition['columnName']] = $condition['value'];
            }
            Util::getValueInFilterDataSourceByName($row, array('NRESTRUTSUPERIOR', 'NRORG', 'DTMESCOMPETENC'), true);
            $filters = Util::createQueryFiltered($row);
            $entidadesEducacionais = $this->estruturasService->getEntidadeEducacional($conditions, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getEntidadeEducacional', $entidadesEducacionais));
            
        } catch (\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		} 
    }
    
    public function getEstruturasEducacionaisAdicionar($request, $response){
        try{
            $conditions = array();
            $row = $request->getFilterCriteria()->getConditions();
            foreach($row as $key => $condition){
                $conditions[$condition['columnName']] = $condition['value'];
            }
            $hierarquias = $conditions['NRHIERARQUIAORG'];
            $conditions['NRHIERARQUIAORG'] = '';
            foreach($hierarquias as $hierarquia){
                if(array_key_exists('NRHIERARQUIAORG', $conditions) && $hierarquia != ''){
                    $conditions['NRHIERARQUIAORG'] = implode(',', $hierarquias);
                }else{
                    $conditions['NRHIERARQUIAORG'] = null;
                }
            }
            Util::getValueInFilterDataSourceByName($row, array('DTMESCOMPETENC', 'NRORG', 'NRTIPOESTRUTURA', 'NRHIERARQUIAORG', 'NRESTRUTSUPERIOR'), true);
            $filters = Util::createQueryFiltered($row);   
            $entidadesEducacionais = $this->estruturasService->getEstruturasEducacionaisAdicionar($conditions['NRORG'], $conditions['DTMESCOMPETENC'], $conditions['NRTIPOESTRUTURA'], $conditions['NRHIERARQUIAORG'], $conditions['NRESTRUTSUPERIOR'], $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getEstruturasEducacionaisAdicionar', $entidadesEducacionais));
                        
        } catch (\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		} 
    }
    
    public function setEntidadeEducacional($request, $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $retorno = $this->estruturasService->setEntidadeEducacional($row);
            $response->addNotification(new Notification("Entidade Educacional ou Desportiva salva com sucesso", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		} 
    }
    
    public function deleteEntidadeEducacional($request, $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $nrorg = $this->hcmParameters->getNrorgTrabalho(); 
            $this->estruturasService->deleteEntidadeEducacional($nrorg, $row['NRHIERARQUIAORG']);
        } catch (\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		} 
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
    public function verificaMovimentacaoeHierarquia(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        try {
            $mensagem = $this->estruturasService->verificaMovimentacaoeHierarquia($row);
            if($mensagem != null){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($mensagem,'E'));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function consultaRF(DTO\Request\Row $row, DTO\Response $response) {
        //var_dump("teste");die;
        $row = $row->getRow();
        $dtmescompetenc = $this->hcmParameters->getDtcompetencia();
        $nrinsjurcan = $row['CDCNPJESTRUT'];

        $params = array(
            "nrorg" => $this->hcmParameters->getNrorgTrabalho(),
            "nrorgPadrao" => $this->hcmParameters->getNrorgPadrao(),
            "dtmescompetenc" => $this->hcmParameters->getDtcompetencia(),
            "nrinsjurcan" => $row['CDCNPJESTRUT']
        );
        $consulta = $this->consultaInscricaoRf->consultaDadoRF($params, $this->entityManager);
        
        $response->addDataSet(new DataSet("CONSULTA", $consulta));
        
    }
}