<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Controller\Crud;
use HCM\PTO\Services\RoteiroUsoFrequenciaService;
use HCM\Util\Messages;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;

class RoteiroUsoFrequenciaController extends Crud {
    
    protected $mainDataSourceName                       = 'getPassosUsoFreq';
    protected $dataSourceGetPeriodoRoteiroUsoFrequencia = 'getPeriodoRoteiroUsoFrequencia';
    protected $parameterBagColumns                      = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRPERIODOAPURACAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, RoteiroUsoFrequenciaService $roteiroUsoFrequenciaService) {
        $this->parameterBag                     = $parameterBag;
        $this->roteiroUsoFrequenciaService      = $roteiroUsoFrequenciaService;
        $this->hcmParameters                    = $hcmParameters;
        parent::__construct($dataSourceManager);
    }
    
    public function getPassosUsoFreq(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->mainDataSourceName);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getPeriodoRoteiroUsoFrequencia(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetPeriodoRoteiroUsoFrequencia);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setUsoFreqDataUltOper(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            $nrPassosUsoFreq    = $row['NRPASSOSUSOFREQ']       ?? null;
            $nrPeriodoApuracao  = $row['NRPERIODOAPURACAO']     ?? null;
            
            $result = $this->roteiroUsoFrequenciaService->setUsoFreqDataUltOper(
                $nrPassosUsoFreq,
                $nrPeriodoApuracao
            );
            
            $dataSet = array('success' => $result);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}