<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\EpiService;
use HCM\Geral\Services\UploadFileService;

class EpiController extends Crud {

    protected $dataSourceName = 'sst_epi_get';

    protected $parameterBagColumns = array('NRORG', 'NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, UploadFileService $uploadFileService,  EpiService $epiService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->epiService = $epiService;
        $this->uploadFileService = $uploadFileService;
        parent::__construct($dataSourceManager);
    }

    public function getEpi(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $newRows = array();
        foreach($dataSet->getRows() as $row){
            $row['NRORG_PADRAO'] = $this->hcmParameters->getNrorgPadrao();
            $row['DSARQIMGEPI'] = $this->epiService->getFoto($row);
            $row['NRANEXO'] = !is_null($row['DSARQIMGEPI']) ? $row['DSARQIMGEPI']['NRANEXO'] : null;
            array_push($newRows, $row);
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('sst_epi_get', $newRows));
    }

    public function setEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            error_reporting(E_ERROR | E_PARSE);
            $this->dataSourceName = "sst_epi";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if(is_array($row['DSARQIMGEPI'])){
                    $row['DSARQIMGEPI'] = $row['DSARQIMGEPI'][0]['path'];
                }
                if (array_key_exists('NREPI', $row)) {
                    if ($row['NREPI'] === '') {
                        $row['NREPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_EPI', 1, 12, true);
                    }
                } else {
                    $row['NREPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_EPI', 1, 12, true);
                }
                if(!array_key_exists('DSARQIMGEPI', $row)){
                    $row['DSARQIMGEPI'] = null;
                }
                if(!array_key_exists('NRPARCNEGOCIO', $row)){
                    $row['NRPARCNEGOCIO'] = null;
                }
                if (!array_key_exists('CDOPERADOR_LOG', $row)) 
                    $row['CDOPERADOR_LOG']= $this->hcmParameters->getCdoperador();
                if(array_key_exists('DSPATH', $row) && !is_null($row['DSPATH'])){
                    $row['NRPARCNEGOCIO'] = $row['NREPI'];
                    $row['IDFOTO'] = 'S';
                    $this->uploadFileService->setGpeAnexos($row);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteEpi(DTO\Request\DataSet $request, DTO\Response $response){
        if($this->epiService->validaDeleteEpi($request->getDataSet()->getRows()[0]['NRORG'],$request->getDataSet()->getRows()[0]['NREPI']))
            parent::delete($request, $response);
        else
	        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Não foi possivel excluir, encontrado registro filho.", DTO\Response\Message::TYPE_ERROR));
        
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function baixarPlanilhaEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $file = $this->epiService->baixarPlanilhaEpi();
                
            if($file != null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $messages = $this->epiService->importarPlanilhaEpi($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
} 